# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio app rdm records administration module."""
from invenio_app_rdm.administration.records.records import (
    DraftAdminListView,
    RecordAdminListView,
)

__all__ = ("RecordAdminListView", "DraftAdminListView")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Tests for the CLI."""

from invenio_access.permissions import system_identity
from invenio_rdm_records.proxies import current_oaipmh_server_service

from invenio_app_rdm.cli import create_fixtures


def test_create_fixtures(app, db, cli_runner):
    """Assert that fixtures are created."""
    result = cli_runner(create_fixtures)
    assert result.exit_code == 0

    service = current_oaipmh_server_service
    res_set = service.search(system_identity, params={"q": f"%"})

    # oai_sets.yaml file left empty
    assert res_set.total == 2



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Communities UI blueprints module."""

from flask import Blueprint, current_app, render_template
from flask_login import current_user
from flask_menu import current_menu
from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)
from invenio_communities.errors import CommunityDeletedError
from invenio_i18n import lazy_gettext as _
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_records_resources.services.errors import PermissionDeniedError

from ..searchapp import search_app_context
from .communities import communities_detail


#
# Error handlers
#
def not_found_error(error):
    """Handler for 'Not Found' errors."""
    return render_template(current_app.config["THEME_404_TEMPLATE"]), 404


def record_tombstone_error(error):
    """Tombstone page."""
    # the RecordDeletedError will have the following properties,
    # while the PIDDeletedError won't
    record = getattr(error, "record", None)
    if (record_ui := getattr(error, "result_item", None)) is not None:
        if record is None:
            record = record_ui._record
        record_ui = UICommunityJSONSerializer().dump_obj(record_ui.to_dict())

    # render a 404 page if the tombstone isn't visible
    if not record.tombstone.is_visible:
        return not_found_error(error)

    # we only render a tombstone page if there is a record with a visible tombstone
    return (
        render_template(
            "invenio_communities/tombstone.html",
            record=record_ui,
        ),
        410,
    )


def record_permission_denied_error(error):
    """Handle permission denier error on record views."""
    if not current_user.is_authenticated:
        # trigger the flask-login unauthorized handler
        return current_app.login_manager.unauthorized()
    return render_template(current_app.config["THEME_403_TEMPLATE"]), 403


def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config["RDM_COMMUNITIES_ROUTES"]

    blueprint = Blueprint(
        "invenio_app_rdm_communities",
        __name__,
        template_folder="../templates",
        static_folder="../static",
    )

    blueprint.add_url_rule(
        routes["community-detail"],
        view_func=communities_detail,
        strict_slashes=False,
    )

    @blueprint.before_app_first_request
    def register_menus():
        """Register community menu items."""
        communities = current_menu.submenu("communities")
        communities.submenu("search").register(
            "invenio_app_rdm_communities.communities_detail",
            text=_("Search"),
            order=1,
            expected_args=["pid_value"],
            **dict(icon="search", permissions=True),
        )

    # Register error handlers
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(CommunityDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)

    # Register context processor
    blueprint.app_context_processor(search_app_context)

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.


def test_static(app, client, es_clear):
    """Check if robots.txt returns a string response."""

    # Save the current static folder to revert after finishing the test.
    app_static_folder = app.static_folder
    # Use as static folder the one of the blueprint we are testing.
    app.static_folder = app.blueprints["invenio_app_rdm"].static_folder

    response = client.get("/robots.txt")

    assert response.status_code == 200
    assert type(response.text) == str
    assert "Disallow: /search" in response.text
    assert "Disallow: /api" in response.text
    assert "Disallow: /administration" in response.text

    app.static_folder = app_static_folder



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 TU Wien.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 6.0 to 7.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 6.0 to 7.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""


from click import secho
from invenio_db import db
from invenio_rdm_records.proxies import current_rdm_records_service
from invenio_rdm_records.records.api import RDMDraft, RDMRecord
from invenio_records.systemfields.relations.errors import InvalidCheckValue


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 6.0 to 7.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """

    def migrate_records_and_parents(record_cls, check_deleted):
        """Migrate records (and parents) from v6 to v7."""
        success = True
        records = []
        for record_metadata in record_cls.model_cls.query.all():
            # skipping deleted drafts because can't be committed
            if check_deleted and record_metadata.is_deleted:
                continue

            record = record_cls(record_metadata.data, model=record_metadata)

            # migrate the parent record to the new schema, if necessary
            new_schema = "local://records/parent-v2.0.0.json"
            if record.parent["$schema"] != new_schema:
                record.parent["$schema"] = new_schema
                record.parent.commit()

            try:
                record.commit()
                records.append(record)

            except Exception as exc:
                # report errors along the way
                success = False
                pid = record.pid.pid_value

                if isinstance(exc, InvalidCheckValue):
                    # this can happen when when e.g. a record's resource type
                    # is set to something that's not depositable
                    secho(
                        f"{record_cls.__name__} '{pid}' has a problem, likely "
                        "with an invalid 'resource_type' value. "
                        "This should be fixable by changing its metadata "
                        "in a way that it can be saved successfully.",
                        fg="red",
                        err=True,
                    )
                else:
                    secho(
                        f"{record_cls.__name__} '{pid}' needs fixing.",
                        fg="red",
                        err=True,
                    )

                secho(f"> Error message: {exc}\n", fg="red", err=True)

        return records, success

    def create_oai_identifiers(records):
        """Create and reserve OAI identifiers for the records.

        This will create OAI PIDs for the records and their drafts.
        We skip drafts that aren't published, because they'll get their
        OAI PIDs upon publication.
        """
        service = current_rdm_records_service
        pid_manager = service.pids.pid_manager
        for record in records:
            # skip records that already have some kind of OAI PID set
            if "oai" in record.pids:
                secho(
                    f"Skipping record '{record.pid.pid_value}', as there's "
                    f"already an OAI PID registered: {record.pids['oai']}",
                    fg="yellow",
                )
                continue

            # create and set the OAI PID for the record
            oai_pid = pid_manager.create(record, "oai", None)
            record.pids["oai"] = oai_pid
            record.commit()

            # set the OAI PID for the draft as well, if there is one
            try:
                draft = service.draft_cls.pid.resolve(
                    record.pid.pid_value, registered_only=False
                )
                if not draft.is_deleted:
                    draft.pids["oai"] = oai_pid
                    draft.commit()

            except Exception:
                pass

            # reserve the OAI PID
            pid_manager.reserve(
                record, "oai", oai_pid["identifier"], oai_pid["provider"]
            )

    def reindex_records(records):
        """Reindex the given records."""
        for record in records:
            current_rdm_records_service.indexer.index(record)

    success = True

    secho("Migrating records and record parents...")
    records, success1 = migrate_records_and_parents(RDMRecord, False)
    success = success and success1

    secho("Migrating drafts...")
    drafts, success2 = migrate_records_and_parents(RDMDraft, True)
    success = success and success2

    # only finish the migration if there have been no errors
    if success:
        try:
            secho("Creating OAI PIDs for records...")
            create_oai_identifiers(records)
            db.session.commit()

            reindex_records(records)
            reindex_records(drafts)
            secho("Records and drafts migrated.", fg="green")

        except Exception as exc:
            success = False
            secho(f"Error while generating OAI PIDs: {exc}\n", fg="red", err=True)

    if not success:
        secho(
            "Upgrade aborted: There have been problems with the above "
            "mentioned records/drafts - please fix them and try the "
            "migration again!",
            fg="yellow",
            err=True,
        )
        db.session.rollback()


if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 10.0 to 11.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 10.0 to 11.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

from click import secho
from invenio_db import db
from invenio_files_rest.models import Bucket, FileInstance


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 10.0 to 11.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """
    secho("Starting data migration...", fg="green")
    buckets = Bucket.query.all()

    for bucket in buckets:
        bucket.default_storage_class = "L"
    db.session.add(bucket)

    file_instances = FileInstance.query.all()

    for fi in file_instances:
        fi.storage_class = "L"
        db.session.add(fi)

    secho("Committing to DB", fg="green")
    db.session.commit()
    secho("Data migration completed.", fg="green")


# if the script is executed on its own, perform the upgrade
if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2021 CERN.
# Copyright (C) 2019-2021 Northwestern University.
# Copyright (C)      2021 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Routes for record-related pages provided by Invenio-App-RDM."""

from functools import wraps

from flask import g, redirect, request, url_for
from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)
from invenio_communities.proxies import current_communities
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_rdm_records.proxies import current_rdm_records
from invenio_records_resources.services.errors import PermissionDeniedError
from sqlalchemy.orm.exc import NoResultFound


def service():
    """Get the record service."""
    return current_rdm_records.records_service


def files_service():
    """Get the record files service."""
    return current_rdm_records.records_service.files


def media_files_service():
    """Get the record files service."""
    return current_rdm_records.records_media_files_service.files


def draft_files_service():
    """Get the record files service."""
    return current_rdm_records.records_service.draft_files


def draft_media_files_service():
    """Get the record files service."""
    return current_rdm_records.records_media_files_service.draft_files


def pass_record_latest(f):
    """Decorate a view to pass the latest version of a record."""

    @wraps(f)
    def view(**kwargs):
        pid_value = kwargs.get("pid_value")
        record_latest = service().read_latest(id_=pid_value, identity=g.identity)
        kwargs["record"] = record_latest
        return f(**kwargs)

    return view


def pass_draft(expand=False):
    """Decorator to retrieve the draft using the record service."""

    def decorator(f):
        @wraps(f)
        def view(**kwargs):
            pid_value = kwargs.get("pid_value")
            try:
                draft = service().read_draft(
                    id_=pid_value,
                    identity=g.identity,
                    expand=expand,
                )
                kwargs["draft"] = draft
                return f(**kwargs)
            except PIDDoesNotExistError:
                # Redirect to /records/:id because users are interchangeably
                # using /records/:id and /uploads/:id when sharing links, so in
                # case a draft doesn't exists, when check if the record exists
                # always.
                return redirect(
                    url_for(
                        "invenio_app_rdm_records.record_detail",
                        pid_value=pid_value,
                    )
                )

        return view

    return decorator


def pass_is_preview(f):
    """Decorate a view to check if it's a preview."""

    @wraps(f)
    def view(**kwargs):
        preview = request.args.get("preview")
        is_preview = False
        if preview == "1":
            is_preview = True
        kwargs["is_preview"] = is_preview
        return f(**kwargs)

    return view


# TODO to be improved as a request arg schema (following the REST views)
def pass_include_deleted(f):
    """Decorate a view to check if it's a include deleted."""

    @wraps(f)
    def view(**kwargs):
        deleted_arg = request.args.get("include_deleted")
        include_deleted = False
        if deleted_arg == "1":
            include_deleted = True
        kwargs["include_deleted"] = include_deleted
        return f(**kwargs)

    return view


def pass_record_from_pid(f):
    """Decorate a view to pass the record from a pid."""

    @wraps(f)
    def view(*args, **kwargs):
        scheme = kwargs.get("pid_scheme")
        pid_value = kwargs.get("pid_value")

        record = service().pids.resolve(
            g.identity,
            pid_value,
            scheme,
        )

        kwargs["record"] = record
        return f(**kwargs)

    return view


def pass_record_or_draft(expand=False):
    """Decorate to retrieve the record or draft using the record service."""

    def decorator(f):
        @wraps(f)
        def view(**kwargs):
            pid_value = kwargs.get("pid_value")
            is_preview = kwargs.get("is_preview")
            include_deleted = kwargs.get("include_deleted", False)
            read_kwargs = {
                "id_": pid_value,
                "identity": g.identity,
                "expand": expand,
            }

            if is_preview:
                try:
                    record = service().read_draft(**read_kwargs)
                except NoResultFound:
                    try:
                        record = service().read(
                            include_deleted=include_deleted, **read_kwargs
                        )
                    except NoResultFound:
                        # If the parent pid is being used we can get the id of the latest record and redirect
                        latest_version = service().read_latest(**read_kwargs)
                        return redirect(
                            url_for(
                                "invenio_app_rdm_records.record_detail",
                                pid_value=latest_version.id,
                                preview=1,
                            )
                        )
            else:
                try:
                    record = service().read(
                        include_deleted=include_deleted, **read_kwargs
                    )
                except NoResultFound:
                    # If the parent pid is being used we can get the id of the latest record and redirect
                    latest_version = service().read_latest(**read_kwargs)
                    return redirect(
                        url_for(
                            "invenio_app_rdm_records.record_detail",
                            pid_value=latest_version.id,
                        )
                    )
            kwargs["record"] = record
            return f(**kwargs)

        return view

    return decorator


def pass_file_item(is_media=False):
    """Decorator to pass a file or media file item using the corresponding service."""

    def decorator(f):
        @wraps(f)
        def view(**kwargs):
            pid_value = kwargs.get("pid_value")
            file_key = kwargs.get("filename")
            is_preview = kwargs.get("is_preview")
            read_kwargs = {
                "id_": pid_value,
                "file_key": file_key,
                "identity": g.identity,
            }
            draft_service = (
                draft_media_files_service if is_media else draft_files_service
            )
            record_service = media_files_service if is_media else files_service

            if is_preview:
                try:
                    item = draft_service().get_file_content(**read_kwargs)
                except NoResultFound:
                    item = record_service().get_file_content(**read_kwargs)
            else:
                item = record_service().get_file_content(**read_kwargs)

            kwargs["file_item"] = item
            return f(**kwargs)

        return view

    return decorator


def pass_file_metadata(f):
    """Decorate a view to pass a file's metadata using the files service."""

    @wraps(f)
    def view(**kwargs):
        pid_value = kwargs.get("pid_value")
        file_key = kwargs.get("filename")
        is_preview = kwargs.get("is_preview")
        read_kwargs = {"id_": pid_value, "file_key": file_key, "identity": g.identity}

        if is_preview:
            try:
                files = draft_files_service().read_file_metadata(**read_kwargs)
            except NoResultFound:
                files = files_service().read_file_metadata(**read_kwargs)
        else:
            files = files_service().read_file_metadata(**read_kwargs)

        kwargs["file_metadata"] = files
        return f(**kwargs)

    return view


def pass_record_files(f):
    """Decorate a view to pass a record's files using the files service."""

    @wraps(f)
    def view(**kwargs):
        is_preview = kwargs.get("is_preview")
        pid_value = kwargs.get("pid_value")
        read_kwargs = {"id_": pid_value, "identity": g.identity}

        try:
            if is_preview:
                try:
                    files = draft_files_service().list_files(**read_kwargs)
                except NoResultFound:
                    files = files_service().list_files(**read_kwargs)
            else:
                files = files_service().list_files(**read_kwargs)

            kwargs["files"] = files

        except PermissionDeniedError:
            # this is handled here because we don't want a 404 on the landing
            # page when a user is allowed to read the metadata but not the
            # files
            kwargs["files"] = None

        return f(**kwargs)

    return view


def pass_record_media_files(f):
    """Decorate a view to pass a record's media files using the files service."""

    @wraps(f)
    def view(**kwargs):
        is_preview = kwargs.get("is_preview")
        pid_value = kwargs.get("pid_value")
        read_kwargs = {"id_": pid_value, "identity": g.identity}

        try:
            if is_preview:
                try:
                    media_files = draft_media_files_service().list_files(**read_kwargs)
                except NoResultFound:
                    media_files = media_files_service().list_files(**read_kwargs)
            else:
                media_files = media_files_service().list_files(**read_kwargs)

            kwargs["media_files"] = media_files

        except PermissionDeniedError:
            # this is handled here because we don't want a 404 on the landing
            # page when a user is allowed to read the metadata but not the
            # files
            kwargs["media_files"] = None

        return f(**kwargs)

    return view


def pass_draft_files(f):
    """Decorate a view to pass a draft's files using the files service."""

    @wraps(f)
    def view(**kwargs):
        try:
            pid_value = kwargs.get("pid_value")
            files = draft_files_service().list_files(id_=pid_value, identity=g.identity)
            kwargs["draft_files"] = files

        except PermissionDeniedError:
            # this is handled here because we don't want a 404 on the landing
            # page when a user is allowed to read the metadata but not the
            # files
            kwargs["draft_files"] = None

        return f(**kwargs)

    return view


def pass_draft_community(f):
    """Decorate to retrieve the community record using the community service.

    Pass the community record or None when creating a new draft and having
    selected a community via the url.
    """

    @wraps(f)
    def view(**kwargs):
        comid = request.args.get("community")
        if comid:
            community = current_communities.service.read(id_=comid, identity=g.identity)
            kwargs["community"] = UICommunityJSONSerializer().dump_obj(
                community.to_dict()
            )

        return f(**kwargs)

    return view
